package com.ipsplm.kafka;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ipsplm.entity.iot.IotDeviceListNew;
import com.ipsplm.entity.iot.IotDeviceRealInfo;
import com.ipsplm.service.IIotDeviceListNewService;
import com.ipsplm.utils.JSONUtils;
import com.ipsplm.utils.RedisUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * @Description kafka消费者监听
 * @Author FanDongqiang
 * @Date 2023/4/26 10:19
 * @Version 1.0
 */
@Component
@Slf4j
public class ConsumerRedisListener {
    private IIotDeviceListNewService iIotDeviceListNewService;

    @Autowired
    private RedisUtils redisUtils;

    // 设备编号List
    private static Map<String, Integer> deviceNumMap = new HashMap<>();

    public ConsumerRedisListener(IIotDeviceListNewService iIotDeviceListNewService) {
        this.iIotDeviceListNewService = iIotDeviceListNewService;
        //IOT设备清单(新）
        List<IotDeviceListNew> deviceListNew = this.iIotDeviceListNewService.list();
        deviceListNew.forEach(dln -> {
            deviceNumMap.put(dln.getDeviceNum(), dln.getWebsocketGroupId());
        });
        String groupId = "shenyang_real_time_" + System.currentTimeMillis();
        System.out.println("kafka消费者组号：" + groupId + "*********");
        System.setProperty("groupId", groupId);
    }

    /**
     * 处理单条消息
     *
     * @param record 消费记录
     */
    @KafkaListener(topics = {"topic_kafka_huarui"}, groupId = "${groupId}", containerFactory = "kafkaListenerContainerFactory")
    public void handleMessage(ConsumerRecord<String, String> record) {
        // 当前时间
        Date currentTime = new Date();
        // 设备实时数据
        IotDeviceRealInfo iotDeviceRealInfo = new IotDeviceRealInfo();
        // 设备互联数据
        String deviceInfo = record.value();
        // 消息时间戳
        long timestamp = record.timestamp();
        if (StrUtil.isNotBlank(deviceInfo) && JSONUtils.isJSONValidate(deviceInfo)) {
            JSONObject deviceInfoJson = JSON.parseObject(deviceInfo);
            // 设备编号
            String deviceNum = deviceInfoJson.getString("__assetId__");
            if (deviceNumMap.keySet().contains(deviceNum)) {
                //设备编号
                iotDeviceRealInfo.setDeviceNum(deviceNum);
                iotDeviceRealInfo.setDeviceStatus(deviceInfoJson.containsKey("Status") ? deviceInfoJson.getInteger("Status") : deviceInfoJson.getInteger("status"));
                iotDeviceRealInfo.setDeviceTime(DateUtil.date(timestamp));
                iotDeviceRealInfo.setDeviceTimeStr(DateUtil.format(DateUtil.date(timestamp), "yyyy-MM-dd HH:mm:ss"));
                if (deviceInfoJson.containsKey("alarm")) {
                    iotDeviceRealInfo.setAlarm(String.valueOf(deviceInfoJson.getInteger("alarm")));
                }
                if (deviceInfoJson.containsKey("AlarmMsg")) {
                    iotDeviceRealInfo.setAlarmMsg(deviceInfoJson.getString("AlarmMsg"));
                }
                Map<String, Object> positionMap = new HashMap<>();
                if (deviceInfoJson.getString("__thingName__").contains("AGV")) {
                    positionMap.put("AGVPoseX", deviceInfoJson.containsKey("AGVPoseX") ? deviceInfoJson.getDouble("AGVPoseX") : null);
                    positionMap.put("AGVPoseY", deviceInfoJson.containsKey("AGVPoseY") ? deviceInfoJson.getDouble("AGVPoseY") : null);
                    positionMap.put("AGVPoseTheta", deviceInfoJson.containsKey("AGVPoseTheta") ? deviceInfoJson.getDouble("AGVPoseTheta") : null);
                    //电流
                    positionMap.put("BatCurrent", deviceInfoJson.containsKey("BatCurrent") ? deviceInfoJson.getDouble("BatCurrent") : null);
                    //电压
                    positionMap.put("BatVoltage", deviceInfoJson.containsKey("BatVoltage") ? deviceInfoJson.getDouble("BatVoltage") : null);
                    //功能状态
                    positionMap.put("BatPowerSupplyStatus", deviceInfoJson.containsKey("BatPowerSupplyStatus") ? deviceInfoJson.getDouble("BatPowerSupplyStatus") : null);
                    //电池电量
                    positionMap.put("BatteryCapacity", deviceInfoJson.containsKey("BatteryCapacity") ? deviceInfoJson.getDouble("BatteryCapacity") : null);
                    //总行驶里程
                    positionMap.put("TotalMileage", deviceInfoJson.containsKey("TotalMileage") ? deviceInfoJson.getDouble("TotalMileage") : null);
                    //日行驶里程
                    positionMap.put("DayMileage", deviceInfoJson.containsKey("DayMileage") ? deviceInfoJson.getDouble("DayMileage") : null);
                    //工作任务状态
                    positionMap.put("TaskStatus", deviceInfoJson.containsKey("TaskStatus") ? deviceInfoJson.getDouble("TaskStatus") : null);
                } else {
                    String[] keys = {"J1", "J2", "J3", "J4", "J5", "J6", "E1","J7","J8","J9","FixName","WeldDetect"};
                    for (String key : keys) {
                        if (deviceInfoJson.containsKey(key)) {
                            positionMap.put(key, deviceInfoJson.get(key));
                        }
                    }
                }
                iotDeviceRealInfo.setPointPosition(JSON.toJSONString(positionMap));
                iotDeviceRealInfo.setTopic(record.topic());
                iotDeviceRealInfo.setPart(String.valueOf(record.partition()));
                iotDeviceRealInfo.setWebsocketGroupId(deviceNumMap.get(deviceNum));
                iotDeviceRealInfo.setGmtModified(currentTime);
                iotDeviceRealInfo.setGmtCreated(currentTime);
            }
        }
        if (iotDeviceRealInfo.getDeviceNum() != null) {
            redisUtils.set("kafka:" + iotDeviceRealInfo.getDeviceNum() + ":" + iotDeviceRealInfo.getPart(), JSON.toJSONString(iotDeviceRealInfo));
        }
    }
}
